// 
// Main.cs
//  
// Author:
//       Jon Thysell <thysell@gmail.com>
// 
// Copyright (c) 2013 Jon Thysell <http://jonthysell.com>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.IO;
using System.Reflection;

using com.jonthysell.Chordious.Core;

namespace com.jonthysell.Chordious.CLI
{
    class MainClass
    {
        public static ChordOptions _chordOptions;

        public static void Main(string[] args)
        {
            Console.WriteLine(AppInfo.ProgramTitle);

            bool showHelp = false;
            bool invalidArguments = false;
            try
            {
                _chordOptions = new ChordOptions();

                if (args.Length == 1 && args[0] == "-?")
                {
                    showHelp = true;
                }
                else if (args.Length == 1 && File.Exists(args[0]))
                {
                    // Input File, Working Directory
                    WriteChordsInFile(args[0]);
                }
                else if (args.Length == 1 && !File.Exists(args[0]))
                {
                    // Input String, Working Directory
                    Chord c = new Chord(args[0]);
                    WriteChord(c);
                }
                else if (args.Length == 2 && File.Exists(args[0]))
                {
                    // Input File, Output Directory
                    WriteChordsInFile(args[0], args[1]);
                }
                else if (args.Length == 2 && !File.Exists(args[0]))
                {
                    // Input String, Output Directory
                    Chord c = new Chord(args[0]);
                    WriteChord(c, args[1]);
                }
                else
                {
                    invalidArguments = true;
                }

                if (invalidArguments)
                {
                    throw new Exception("Invalid command-line arguments.");
                }

            }
            catch (Exception ex)
            {
                Console.Error.WriteLine(ex.Message);
                Console.WriteLine("Use -? to show help.");
            }
            finally
            {
                if (showHelp)
                {
                    ShowHelp();
                }
            }
        }

        public static void ShowHelp()
        {
            Console.WriteLine();
            Console.WriteLine("{0}\t{1}", "-?", "Show this help");
            Console.WriteLine();
            Console.WriteLine("Usage:");
            Console.WriteLine();
            Console.WriteLine("{0}.exe [infile] [outfolder]", AppInfo.ProgramName);
            Console.WriteLine("{0}.exe [instring] [outfolder]", AppInfo.ProgramName);
        }

        public static void WriteChord(Chord chordChart, string outputFolder = "./")
        {
            if (null == chordChart)
            {
                throw new ArgumentNullException("chordChart");
            }

            if (String.IsNullOrEmpty(outputFolder))
            {
                outputFolder = "./";
            }

            string fileName = Path.Combine(outputFolder, chordChart.FileName + ".svg");

            using (StreamWriter sw = new StreamWriter(fileName))
            {
                Console.WriteLine("Writing chord {0} to {1}...", chordChart.Title, fileName);
                sw.Write(chordChart.ToSvg(_chordOptions));
            }
        }

        public static void WriteChordsInFile(string inputFile, string outputFolder = "./")
        {
            if (String.IsNullOrEmpty(inputFile))
            {
                throw new ArgumentNullException("inputFile");
            }

            if (String.IsNullOrEmpty(outputFolder))
            {
                outputFolder = "./";
            }

            Console.WriteLine("Reading {0}...", inputFile);

            ChordDocument doc = new ChordDocument(inputFile);

            doc.ExportChords(outputFolder);
        }
    }
}