// 
// ChordOptionsTest.cs
//  
// Author:
//       Jon Thysell <thysell@gmail.com>
// 
// Copyright (c) 2013 Jon Thysell <http://jonthysell.com>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;

using NUnit.Framework;

using com.jonthysell.Chordious.Core;

namespace com.jonthysell.Chordious.CoreTest
{
    [TestFixture()]
	public class ChordOptionsTest
    {
        [TearDown()]
        public virtual void TearDown()
        {
            _chordOptions = null;
        }

        #region New Default ChordOptions

        [Test()]
        public void NewDefaultChordOptions()
        {
            _chordOptions = new ChordOptions();
            Assert.IsNotNull(_chordOptions);
        }

        [Test()]
        public void NewDefaultChordOptionsUsesDefaults()
        {
            _chordOptions = new ChordOptions();
            Assert.IsNotNull(_chordOptions);

            Assert.AreEqual(ChordOptions.DefaultWidth, _chordOptions.Width);
            Assert.AreEqual(ChordOptions.DefaultHeight, _chordOptions.Height);
            Assert.AreEqual(ChordOptions.DefaultStrokeWidth, _chordOptions.StrokeWidth);
            Assert.AreEqual(ChordOptions.DefaultMargin, _chordOptions.Margin);
            Assert.AreEqual(ChordOptions.DefaultFontSize, _chordOptions.FontSize);
            Assert.AreEqual(ChordOptions.DefaultFontFamily, _chordOptions.FontFamily);
            Assert.AreEqual(ChordOptions.DefaultFontStyle, _chordOptions.FontStyle);
            Assert.AreEqual(ChordOptions.DefaultBarreType, _chordOptions.BarreType);
            Assert.AreEqual(ChordOptions.DefaultOpenStringType, _chordOptions.OpenStringType);
            Assert.AreEqual(ChordOptions.DefaultFullBarres, _chordOptions.FullBarres);
        }

        #endregion

        #region New ChordOptions Via Params Tests

        [Test()]
        public void NewChordOptionsViaParamsTest()
        {
            _chordOptions = new ChordOptions(_width, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres);
            Assert.IsNotNull(_chordOptions);
        }

        [Test()]
        public void NewChordOptionsViaParamsValidPropertiesTest()
        {
            _chordOptions = new ChordOptions(_width, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres);
            Assert.IsNotNull(_chordOptions);

            Assert.AreEqual(_width, _chordOptions.Width);
            Assert.AreEqual(_height, _chordOptions.Height);
            Assert.AreEqual(_strokeWidth, _chordOptions.StrokeWidth);
            Assert.AreEqual(_margin, _chordOptions.Margin);
            Assert.AreEqual(_fontSize, _chordOptions.FontSize);
            Assert.AreEqual(_fontFamily, _chordOptions.FontFamily);
            Assert.AreEqual(_barreType, _chordOptions.BarreType);
            Assert.AreEqual(_openStringType, _chordOptions.OpenStringType);
            Assert.AreEqual(_fullBarres, _chordOptions.FullBarres);
        }

        [Test()]
        public void NewChordOptionsViaInvalidParamsTest()
        {
            double[] invalidValues = { 0, -1, Double.MinValue };

            foreach (double iv in invalidValues)
            {
                Assert.Catch<ArgumentOutOfRangeException>(() => new ChordOptions(iv, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres));
                Assert.Catch<ArgumentOutOfRangeException>(() => new ChordOptions(_width, iv, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres));
                Assert.Catch<ArgumentOutOfRangeException>(() => new ChordOptions(_width, _height, iv, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres));
                Assert.Catch<ArgumentOutOfRangeException>(() => new ChordOptions(_width, _height, _strokeWidth, iv, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres));
                Assert.Catch<ArgumentOutOfRangeException>(() => new ChordOptions(_width, _height, _strokeWidth, _margin, iv, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres));
            }

            string[] invalidStrings = { null, "" };

            foreach (string iv in invalidStrings)
            {
                Assert.Catch<ArgumentNullException>(() => new ChordOptions(_width, _height, _strokeWidth, _margin, _fontSize, iv, _fontStyle, _barreType, _openStringType, _fullBarres));
            }
        }

        #endregion

        #region New ChordOptions Via OptionLine Tests

        [Test()]
        public void NewChordOptionsViaOptionLineTest()
        {
            string optionLine = String.Format("%{0};{1};{2};{3};{4};{5};{6};{7};{8};{9}", _width, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres);
            _chordOptions = new ChordOptions(optionLine);
            Assert.IsNotNull(_chordOptions);
        }

        [Test()]
        public void NewChordOptionsViaOptionLineValidPropertiesTest()
        {
            string optionLine = String.Format("%{0};{1};{2};{3};{4};{5};{6};{7};{8};{9}", _width, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres);

            _chordOptions = new ChordOptions(optionLine);
            Assert.IsNotNull(_chordOptions);

            Assert.AreEqual(_width, _chordOptions.Width);
            Assert.AreEqual(_height, _chordOptions.Height);
            Assert.AreEqual(_strokeWidth, _chordOptions.StrokeWidth);
            Assert.AreEqual(_margin, _chordOptions.Margin);
            Assert.AreEqual(_fontSize, _chordOptions.FontSize);
            Assert.AreEqual(_fontFamily, _chordOptions.FontFamily);
            Assert.AreEqual(_barreType, _chordOptions.BarreType);
            Assert.AreEqual(_openStringType, _chordOptions.OpenStringType);
            Assert.AreEqual(_fullBarres, _chordOptions.FullBarres);
        }

        [Test()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NewChordOptionsNullOptionLineTest()
        {
            _chordOptions = new ChordOptions(null);
        }

        [Test()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void NewChordOptionsEmptyOptionLineTest()
        {
            _chordOptions = new ChordOptions("");
        }

        #endregion

        #region Property Tests

        [Test()]
        public void InvalidPropertyTests()
        {
            _chordOptions = new ChordOptions(_width, _height, _strokeWidth, _margin, _fontSize, _fontFamily, _fontStyle, _barreType, _openStringType, _fullBarres);

            double[] invalidValues = { 0, -1, Double.MinValue };

            foreach (double iv in invalidValues)
            {
                Assert.Catch<ArgumentOutOfRangeException>(() => _chordOptions.Width =  iv);
                Assert.Catch<ArgumentOutOfRangeException>(() => _chordOptions.Height =  iv);
                Assert.Catch<ArgumentOutOfRangeException>(() => _chordOptions.StrokeWidth =  iv);
                Assert.Catch<ArgumentOutOfRangeException>(() => _chordOptions.Margin =  iv);
                Assert.Catch<ArgumentOutOfRangeException>(() => _chordOptions.FontSize =  iv);
            }

            string[] invalidStrings = { null, "" };

            foreach (string iv in invalidStrings)
            {
                Assert.Catch<ArgumentNullException>(() => _chordOptions.FontFamily =  iv);
            }
        }

        #endregion

        protected ChordOptions _chordOptions;
        protected const double _width = 100;
        protected const double _height = 400;
        protected const double _strokeWidth = 1;
        protected const double _margin = 50;
        protected const double _fontSize = 32;
        protected const string _fontFamily = "sans";
        protected const FontStyle _fontStyle = FontStyle.Bold;
        protected const BarreType _barreType = BarreType.Arc;
        protected const OpenStringType _openStringType = OpenStringType.Circle;
        protected const bool _fullBarres = true;
    }
}