//
// ChordOptionsDialog.cs
//  
// Author:
//       Jon Thysell <thysell@gmail.com>
// 
// Copyright (c) 2013 Jon Thysell <http://jonthysell.com>
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using Pango;

namespace com.jonthysell.Chordious.GtkUI
{
    public partial class ChordOptionsDialog : Gtk.Dialog
    {
        public ChordOptionsDialog(MainWindowVM vm)
        {
            if (null == vm)
            {
                throw new ArgumentNullException("vm");
            }

            _vm = vm;

            this.Build();

            Sensitive = false;
            LoadFontNames();
            LoadFromVM();
            Sensitive = true;
        }

        private MainWindowVM _vm;
        private List<string> _fontNames;

        private void LoadFontNames()
        {
            _fontNames = new List<string>();

            foreach (FontFamily fontFamily in PangoContext.Families)
            {
                _fontNames.Add(fontFamily.Name);
            }

            _fontNames.Sort();

            foreach (string fontName in _fontNames)
            {
                _fontFamilyComboBox.AppendText(fontName);
            }
        }

        private void LoadFromVM()
        {
            _widthSpinButton.Value = Double.Parse(_vm.ChordOptionsWidth);
            _heightSpinButton.Value = Double.Parse(_vm.ChordOptionsHeight);
            _strokeWidthSpinButton.Value = Double.Parse(_vm.ChordOptionsStrokeWidth);
            _marginSpinButton.Value = Double.Parse(_vm.ChordOptionsMargin);

            _fontSizeSpinButton.Value = Double.Parse(_vm.ChordOptionsFontSize);

            int fontNameIndex = _fontNames.BinarySearch(_vm.ChordOptionsFontFamily, StringComparer.CurrentCultureIgnoreCase);
            _fontFamilyComboBox.Active = fontNameIndex >= 0 ? fontNameIndex : 0;

            _fontStyleComboBox.Active = _vm.ChordOptionsFontStyleIndex;
            _barreTypeComboBox.Active = _vm.ChordOptionsBarreTypeIndex;
            _openStringTypeComboBox.Active = _vm.ChordOptionsOpenStringTypeIndex;
            _fullBarresCheckButton.Active = _vm.ChordOptionsFullBarres;
        }

        protected void OnWidthSpinButtonChanged(object sender, System.EventArgs e)
        {
            if (_widthSpinButton.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsWidth = _widthSpinButton.Text;
            }
        }

        protected void OnHeightSpinButtonChanged(object sender, System.EventArgs e)
        {
            if (_heightSpinButton.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsHeight = _heightSpinButton.Text;
            }
        }

        protected void OnStrokeWidthSpinButtonChanged(object sender, System.EventArgs e)
        {
            if (_strokeWidthSpinButton.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsStrokeWidth = _strokeWidthSpinButton.Text;
            }
        }

        protected void OnMarginSpinButtonChanged(object sender, System.EventArgs e)
        {
            if (_marginSpinButton.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsMargin = _marginSpinButton.Text;
            }
        }

        protected void OnFontSizeSpinButtonChanged(object sender, System.EventArgs e)
        {
            if (_fontSizeSpinButton.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsFontSize = _fontSizeSpinButton.Text;
            }
        }

        protected void OnFontFamilyComboBoxChanged(object sender, System.EventArgs e)
        {
            if (_fontFamilyComboBox.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsFontFamily = _fontFamilyComboBox.ActiveText;
            }
        }

        protected void OnFontStyleComboBoxChanged(object sender, System.EventArgs e)
        {
            if (_fontStyleComboBox.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsFontStyleIndex = _fontStyleComboBox.Active;
            }
        }

        protected void OnBarreTypeComboBoxChanged(object sender, System.EventArgs e)
        {
            if (_barreTypeComboBox.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsBarreTypeIndex = _barreTypeComboBox.Active;
            }
        }

        protected void OnOpenStringTypeComboBoxChanged(object sender, System.EventArgs e)
        {
            if (_openStringTypeComboBox.State == Gtk.StateType.Normal)
            {
                _vm.ChordOptionsOpenStringTypeIndex = _openStringTypeComboBox.Active;
            }
        }

        protected void OnFullBarresCheckButtonToggled(object sender, System.EventArgs e)
        {
            if (_fullBarresCheckButton.State != Gtk.StateType.Insensitive)
            {
                _vm.ChordOptionsFullBarres = _fullBarresCheckButton.Active;
            }
        }
    }
}