# Chordious installer
# based on scripts at http://nsis.sourceforge.net/Simple_tutorials

!include "MUI2.nsh"
!include "LogicLib.nsh"
!include "FileFunc.nsh"

!define PRODUCT_NAME "Chordious"
!define PRODUCT_PUBLISHER "Jon Thysell"
!define PRODUCT_WEB_SITE "http://chordious.com/"
!define PRODUCT_UNINST_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PRODUCT_NAME}"
!define PRODUCT_UNINST_ROOT_KEY "HKLM"

Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
OutFile "${PRODUCT_NAME}_${PRODUCT_VERSION}_WindowsSetup.exe"
InstallDir "$PROGRAMFILES\Chordious"
ShowInstDetails show
ShowUnInstDetails show

DirText "Choose an installation directory"
 
RequestExecutionLevel admin

# Interface

!insertmacro MUI_PAGE_WELCOME
!insertmacro MUI_PAGE_LICENSE "..\..\Doc\license.txt"
!insertmacro MUI_PAGE_DIRECTORY
!insertmacro MUI_PAGE_INSTFILES
!define MUI_FINISHPAGE_RUN "$INSTDIR\StartChordious.cmd"
!insertmacro MUI_PAGE_FINISH

!insertmacro MUI_UNPAGE_WELCOME
!insertmacro MUI_UNPAGE_CONFIRM
!insertmacro MUI_UNPAGE_INSTFILES
!insertmacro MUI_UNPAGE_FINISH

!insertmacro MUI_LANGUAGE "English"

# start default section
Section "Install"

    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayName" "$(^Name)"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString" "$\"$INSTDIR\uninstall.exe$\""
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "QuietUninstallString" "$\"$INSTDIR\uninstall.exe$\" /S"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayIcon" "$INSTDIR\chordious.ico"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayVersion" "${PRODUCT_VERSION}"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "URLInfoAbout" "${PRODUCT_WEB_SITE}"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "Publisher" "${PRODUCT_PUBLISHER}"
    WriteRegStr ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "InstallLocation" "$INSTDIR\"
    WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoModify" "1"
    WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoRepair" "1"
 
    # set the installation directory as the destination for the following actions
    SetOutPath "$INSTDIR"

    # install the main files
    File "..\Chordious.CLI\bin\Release\Chordious.CLI.exe"
    File "..\Chordious.GtkUI\bin\Release\Chordious.Core.dll"
    File "..\Chordious.GtkUI\bin\Release\Chordious.exe"
    File "..\Chordious.GtkUI\bin\Release\sample.txt"
    File "..\Chordious.GtkUI\bin\Release\builtin.xml"
    File "libcroco-0.6-3.dll"
    File "librsvg-2-2.dll"
    File "rsvg2-sharp.dll"
    File "rsvg2-sharp.dll.config"
    File "StartChordious.cmd"
    File "chordious.ico"
    File "..\..\Doc\license.txt"
    File "..\..\Doc\readme.txt"

    # install the icons
    SetOutPath "$INSTDIR\icons"

    File "..\Chordious.GtkUI\bin\Release\icons\arrow_down.png"
    File "..\Chordious.GtkUI\bin\Release\icons\arrow_up.png"
    File "..\Chordious.GtkUI\bin\Release\icons\comment.png"
    File "..\Chordious.GtkUI\bin\Release\icons\delete_2.png"
    File "..\Chordious.GtkUI\bin\Release\icons\eraser.png"
    File "..\Chordious.GtkUI\bin\Release\icons\flag_mark_gray.png"
    File "..\Chordious.GtkUI\bin\Release\icons\flag_mark_green.png"
    File "..\Chordious.GtkUI\bin\Release\icons\logo32.png"
    File "..\Chordious.GtkUI\bin\Release\icons\logo64.png"
    File "..\Chordious.GtkUI\bin\Release\icons\options_2.png"
    File "..\Chordious.GtkUI\bin\Release\icons\pencil_edit.png"
    File "..\Chordious.GtkUI\bin\Release\icons\plus_2.png"
    File "..\Chordious.GtkUI\bin\Release\icons\refresh.png"
    File "..\Chordious.GtkUI\bin\Release\icons\save_diskette_floppy_disk.png"
    File "..\Chordious.GtkUI\bin\Release\icons\search_lense.png"

    # Fixed estimated size in add/remove programs

    ${GetSize} "$INSTDIR" "/S=0K" $0 $1 $2
    IntFmt $0 "0x%08X" $0
    WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "EstimatedSize" "$0"
 
    # create the uninstaller
    WriteUninstaller "$INSTDIR\uninstall.exe"
 
    # Create shortcuts
    CreateDirectory "$SMPROGRAMS\Chordious"
    CreateShortCut "$SMPROGRAMS\Chordious\Chordious.lnk" "$INSTDIR\StartChordious.cmd" "" "$INSTDIR\chordious.ico"
    CreateShortCut "$SMPROGRAMS\Chordious\Readme.lnk" "$INSTDIR\readme.txt"
    CreateShortCut "$SMPROGRAMS\Chordious\License.lnk" "$INSTDIR\license.txt"
    CreateShortCut "$SMPROGRAMS\Chordious\Uninstall.lnk" "$INSTDIR\uninstall.exe"

SectionEnd
 
# uninstaller section start
Section "uninstall"
 
    # first, delete the uninstaller
    Delete "$INSTDIR\uninstall.exe"
 
    # second, remove the links from the start menu
    Delete "$SMPROGRAMS\Chordious\Uninstall.lnk"
    Delete "$SMPROGRAMS\Chordious\Chordious.lnk"
    Delete "$SMPROGRAMS\Chordious\Readme.lnk"
    Delete "$SMPROGRAMS\Chordious\License.lnk"
    RMDir "$SMPROGRAMS\Chordious"

    Delete "$INSTDIR\Chordious.CLI.exe"
    Delete "$INSTDIR\Chordious.Core.dll"
    Delete "$INSTDIR\Chordious.exe"
    Delete "$INSTDIR\sample.txt"
    Delete "$INSTDIR\builtin.xml"
    Delete "$INSTDIR\libcroco-0.6-3.dll"
    Delete "$INSTDIR\librsvg-2-2.dll"
    Delete "$INSTDIR\rsvg2-sharp.dll"
    Delete "$INSTDIR\rsvg2-sharp.dll.config"
    Delete "$INSTDIR\StartChordious.cmd"
    Delete "$INSTDIR\chordious.ico"
    Delete "$INSTDIR\license.txt"
    Delete "$INSTDIR\readme.txt"

    Delete "$INSTDIR\icons\arrow_down.png"
    Delete "$INSTDIR\icons\arrow_up.png"
    Delete "$INSTDIR\icons\comment.png"
    Delete "$INSTDIR\icons\delete_2.png"
    Delete "$INSTDIR\icons\eraser.png"
    Delete "$INSTDIR\icons\flag_mark_gray.png"
    Delete "$INSTDIR\icons\flag_mark_green.png"
    Delete "$INSTDIR\icons\logo32.png"
    Delete "$INSTDIR\icons\logo64.png"
    Delete "$INSTDIR\icons\options_2.png"
    Delete "$INSTDIR\icons\pencil_edit.png"
    Delete "$INSTDIR\icons\plus_2.png"
    Delete "$INSTDIR\icons\refresh.png"
    Delete "$INSTDIR\icons\save_diskette_floppy_disk.png"
    Delete "$INSTDIR\icons\search_lense.png"

    RMDir "$INSTDIR\icons"
    RMDir "$INSTDIR"

    DeleteRegKey ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}"
 
# uninstaller section end
SectionEnd

# Check for admin before install
Function .onInit
UserInfo::GetAccountType
pop $0
${If} $0 != "admin" ;Require admin rights on NT4+
        MessageBox mb_iconstop "Administrator rights required!"
        SetErrorLevel 740 ;ERROR_ELEVATION_REQUIRED
        Quit
${EndIf}

ClearErrors
ReadRegStr $0 HKLM "SOFTWARE\Novell\Mono" "DefaultCLR"

# Check for Mono install
${If} ${Errors}
    ClearErrors
    ReadRegStr $0 HKLM "SOFTWARE\Wow6432Node\Novell\Mono" "DefaultCLR"
    ${If} ${Errors}
        MessageBox mb_iconstop "Mono not detected! Please install Mono first and try again."
        SetErrorLevel 2
        Quit
    ${EndIf}
${EndIf}

FunctionEnd
